#!/usr/bin/perl

# Script to extract species names and country  
# areas from the World Spider Catalog 
# WH Piel (cc) 2017

use strict;
use HTML::Entities ();
require LWP::UserAgent;

# The important hash here is %speciesHsh, where the keys are 
# each species name and the values are an array of area names
my %countryHsh;
my %speciesHsh;
my %familyHsh;
my %generaHsh;

# First build a list of URLs to family pages
my $spiders_url = 'http://www.wsc.nmbe.ch/families';
my $efetch_result = &urlFetch($spiders_url);
my @family_titles = ($efetch_result =~ m/<a title="Classic view" href="(\/family\/\d+\/\w+)">Catalog<\/a>/g);

# Parse each family page
foreach my $fam ( @family_titles ) {
	my $family_name;
	if ($fam =~ m/^\/family\/\d+\/(\w+)$/) {
		$family_name = $1;
		print "$family_name\n";
		my $fam_url = 'http://www.wsc.nmbe.ch'."$fam";
		push @{ $familyHsh{ $family_name } }, parseFamily( $fam_url, $family_name );
	}
	# Uncomment to test just a few families:
	# last if ($family_name eq "Araneidae");
}

print "List of Areas\n";
foreach my $cntr ( keys(%countryHsh) ) {
	print "$cntr has " . $countryHsh{$cntr} . " species\n";
}
print "\n\n";

# Dump a tab-separated text file of family, genus, species, and areas
open my $fo, '>', "output.txt" or die $!;
	for my $family ( keys %familyHsh ) {
		for my $i ( 0 .. $#{ $familyHsh{$family} } ) {
			my $genus = $familyHsh{$family}[$i];
			for my $j ( 0 .. $#{ $generaHsh{$genus} } ) {
				my $species = $generaHsh{$genus}[$j];
				print $fo "$family\t$genus\t$species\t". join ("\t", @{ $speciesHsh{$species} }) ."\n";
			}
		}
	}
close $fo;

# Subroutine fetches a family page and extracts 
# each species and any areas listed with each species 
#==============================================================
sub parseFamily {
	my $fam_url = shift;
	my $family_name = shift;

	my %genera;
	my $efetch_result = &urlFetch($fam_url);
	
	# Uncomment for debugging
	# open my $fo, '>', "$family_name.txt" or die $!;
	# 	print $fo "$efetch_result\n";
	# close $fo;

	my @species_titles = ($efetch_result =~ m/<div class="speciesTitle">(.+)/g);
	my @species_text = ($efetch_result =~ m/<div class="speciesTitle">((.+\n){4,8}).+<\/div>/g);
	@species_text = grep(/^<a\n\s+ href="\/species\//, @species_text); 

	if ( eval(@species_titles) == eval(@species_text) ) {
		print "   Species found: ". eval(@species_titles)."\n";
	} else {
		print "   WARNING: ". eval(@species_titles) . " vs. " . eval(@species_text) . "\n";
	}

	my $i = 1;
	foreach my $sp_txt ( @species_text ) {
		my ($species_name, $countries, $urn, $ending);
		my @countrylist = ();
		if ($sp_txt =~ m/<a\n\s+ href="\/species\/\d+\/[A-Za-z_-]+"><strong><i>([A-Z].+)<\/i><\/strong><\/a>/) {
			
			$species_name = $1;
			if ($species_name =~ m/^([^ ]+) /){
				push @{ $generaHsh{ $1 } }, "$species_name";
				$genera{ $1 }++;
			}
			
			# Capture the country areas
			if ($sp_txt =~ m/ \| ([^\n]+)\n/) {
				$countries = $1;
			}
			
			if ($sp_txt =~ m/(urn:lsid:nmbe.ch:spidersp:\d+)/) {
				$urn = $1;
			}
			
			if ( $countries =~ m/^([^(]+) \(([^)]+)\)$/ ) {
				$ending = $2;
				$countries = $1;
			}
			$countries =~ s/\. Introduced/, Introduced/;
			
			@countrylist = split(/, /, $countries);
			push (@countrylist, $ending) if ( $ending );
			foreach (@countrylist) {
				$countryHsh{$_}++;
			}
			push @{ $speciesHsh{ $species_name } }, @countrylist;

		} else {
			print "SPECIES NOT FOUND!! \n$sp_txt\n\n";
		}
	}
	return keys(%genera);
}


# Subroutine returns raw HTML using the GET protocol
#==============================================================
sub urlFetch {
	my $efetch = shift;

	my $ua = new LWP::UserAgent;
	my $request = new HTTP::Request('GET', $efetch);
	my $response = $ua->request($request);
	my $efetch_result = $response->content;
	$efetch_result =~ s/\222/'/g;
	return($efetch_result);
}


